# coding=UTF-8
"""
日期、时间处理工具类
"""

import time
import datetime
import calendar

DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"
TIME_FORMAT = "%H:%M:%S"


def generate_10timestamp():
    """根据当前时间获取时间戳，返回整数(十位)"""
    return int(time.time())


def generate_13timestamp():
    """
    毫秒级时间戳(十三位)
    """
    return int(time.time() * 1000)


# 当前日期  格式%Y-%m-%d %H:%M:%S
def curDatetime():
    return datetime.datetime.strftime(datetime.datetime.now(), DATETIME_FORMAT)


# 当前日期  格式%Y-%m-%d
def curDate():
    return datetime.date.today()


# 当前时间  格式%H:%M:%S
def curTime():
    return time.strftime(TIME_FORMAT)


# 秒转日期
def seconds_2Datetime(seconds):
    return time.strftime(DATETIME_FORMAT, time.localtime(seconds))


# 毫秒转日期
def milis_2datetime(milix):
    return time.strftime(DATETIME_FORMAT, time.localtime(milix // 1000))


# 日期转13位毫秒
def datetime_2_13milis(datetimestr):
    strf = time.strptime(datetimestr, DATETIME_FORMAT)
    return int(time.mktime(strf)) * 1000


# 日期转10位毫秒
def datetime_2_10milis(datetimestr):
    strf = time.strptime(datetimestr, DATETIME_FORMAT)
    return int(time.mktime(strf))


# 当前年
def curYear():
    return datetime.datetime.now().year


# 当前月
def curMonth():
    return datetime.datetime.now().month


# 当前日
def curDay():
    return datetime.datetime.now().day


# 当前时
def curHour():
    return datetime.datetime.now().hour


# 当前分
def curMinute():
    return datetime.datetime.now().minute


# 当前秒
def curSecond():
    return datetime.datetime.now().second


# 星期几
def curWeek():
    return datetime.datetime.now().weekday()


def date_2datetime(dt):
    """
    date_2datetime(datetime.date(2013,7,13))
    >>> datetime.datetime(2013,7,13,0,0)
    """
    return datetime.datetime(dt.year, dt.month, dt.day)


def format_datetime(dt):
    """将mongodb保存的时间转格式，去除毫秒精度"""
    if not dt:
        return None
    return datetime.datetime(dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second)


def datetime_2string(dt=None, fmt=DATETIME_FORMAT):
    """将datetime类型转换为字符串"""
    if not dt:
        dt = datetime.datetime.now()
    return datetime.datetime.strftime(dt, fmt)


def string_2datetime(dt_str, fmt=DATETIME_FORMAT):
    """
    将字符串转换为datetime类型
    >>> string_2datetime("2011-09-07 12:30:09")
    datetime.datetime(2011, 9, 7, 12, 30, 9)
    """
    if not dt_str:
        return None
    return datetime.datetime.strptime(dt_str, fmt)


def string_2datetime2(dt_str, fmt='%Y-%m-%d'):
    return string_2datetime(dt_str[:10], fmt)


def string_2date(dt_str):
    """将字符串转换为date类型"""
    return datetime.datetime.strptime(dt_str, '%Y-%m-%d').date()


def get_start_datetime(dt=None):
    """根据日期、或时间取得当日00:00:00的时间；参数可以是date或datetime类型"""
    start = None
    if not dt:
        dt = datetime.date.today()
    if isinstance(dt, datetime.date):
        dt_str = dt.strftime("%Y-%m-%d") + " 00:00:00"
        start = datetime.datetime.strptime(dt_str, DATETIME_FORMAT)
    return start


def get_end_datetime(dt=None):
    """根据日期、或时间取得当日23:59:59的时间；参数可以是date或datetime类型"""
    end = None
    if not dt:
        dt = datetime.date.today()
    if isinstance(dt, datetime.date):
        dt_str = dt.strftime("%Y-%m-%d") + " 23:59:59"
        end = datetime.datetime.strptime(dt_str, DATETIME_FORMAT)
    return end


def get_start_hour(dt=None):
    """根据日期、或时间取得该小时00:00的时间；参数可以是date或datetime类型"""
    start = None
    if not dt:
        dt = datetime.date.today()
    if isinstance(dt, datetime.date):
        dt_str = dt.strftime("%Y-%m-%d %H") + ":00:00"
        start = datetime.datetime.strptime(dt_str, DATETIME_FORMAT)
    return start


def get_end_hour(dt=None):
    """根据日期、或时间取得该小时59:59的时间；参数可以是date或datetime类型"""
    end = None
    if not dt:
        dt = datetime.date.today()
    if isinstance(dt, datetime.date):
        dt_str = dt.strftime("%Y-%m-%d %H") + ":59:59"
        end = datetime.datetime.strptime(dt_str, DATETIME_FORMAT)
    return end


def get_time_delta(begin_dt, end_dt, time_fmt=None):
    """获取时间差，根据time_fmt返回数据，如果time_fmt=None，则返回days,hours,minutes,seconds"""
    days, hours, minutes, seconds, left_seconds = None, None, None, None, 0
    if isinstance(begin_dt, datetime.date) and isinstance(end_dt, datetime.date):
        days = (end_dt - begin_dt).days
        left_seconds = (end_dt - datetime.timedelta(days=days) - begin_dt).seconds
        hours = left_seconds / 3600
        tmp_seconds = left_seconds - hours * 3600
        minutes = tmp_seconds / 60
        seconds = tmp_seconds - minutes * 60

    if time_fmt == "DAYS":
        return days
    elif time_fmt == "HOURS":
        return days * 24 + hours
    elif time_fmt == "MINUTES":
        return (days * 24 + hours) * 60 + minutes
    elif time_fmt == "SECONDS":
        return (days * 24 * 3600) + left_seconds
    else:
        return days, hours, minutes, seconds


def time_is_someday(dt, days=0):
    """判断datetime或date是否是哪一天，默认是今天"""
    if not dt:
        return False
    someday = datetime.date.today() - datetime.timedelta(days=days)
    if isinstance(dt, datetime.datetime):
        return dt.date() == someday
    elif isinstance(dt, datetime.date):
        return dt == someday
    return False


def time_is_ndays_interval(dt, ndays):
    """计算datetime或date与今天是否相差n天，ndays为正数表示相差过去n天，为负数表示相差今后n天"""
    if not dt:
        return False
    before_ndays = datetime.date.today() - datetime.timedelta(days=ndays)
    if isinstance(dt, datetime.datetime):
        return dt.date() <= before_ndays
    elif isinstance(dt, datetime.date):
        return dt <= before_ndays
    return False


def time_is_recent(dt, **kwargs):
    """这个函数其实跟上面的类似，只不过能够支持对hour、second进行判断"""
    if dt and isinstance(dt, datetime.datetime) and kwargs and (
            not set(kwargs.keys()) - {'days', 'hours', 'minutes', 'seconds'}):
        last_time = datetime.datetime.now() - datetime.timedelta(**kwargs)
        return dt >= last_time
    else:
        return False


def time_humanize(dt):
    """将时间转换为"n天前"，使其更人性化，增加可读性"""
    result = ""
    if dt:
        time_ = ""
        if isinstance(dt, datetime.datetime):
            time_ = dt.strftime('%H:%M')
            dt = dt.date()
        days = (datetime.date.today() - dt).days
        if days == 0:
            result = "今天 %s" % time_
        elif days == 1:
            result = "昨天 %s" % time_
        elif days == 2:
            result = "前天 %s" % time_
        elif days > 2:
            result = "%s天前" % days
        return result
    return dt


def days_diff_interval(ed):
    """获取日期与当前日期相差的天数"""
    if not ed:
        return -1
    bd = datetime.date.today()
    oneday = datetime.timedelta(days=1)
    count = 0
    if ed > bd:
        while bd != ed:
            ed = ed - oneday
            count += 1
        return count
    elif ed == bd:
        return 0
    else:
        while bd != ed:
            ed = ed + oneday
            count -= 1
        return count


def get_1st_of_last_month(date):
    """
    获取上个月第一天的日期
    :return: 返回日期
    """
    if date:
        today = date
    else:
        today = datetime.datetime.today()
    year = today.year
    month = today.month
    if month == 1:
        month = 12
        year -= 1
    else:
        month -= 1
    res = datetime.datetime(year, month, 1)
    return res


def get_1st_of_next_month(date):
    """
    获取下个月的1号的日期
    :return: 返回日期
    """
    if date:
        today = date
    else:
        today = datetime.datetime.today()
    year = today.year
    month = today.month
    if month == 12:
        month = 1
        year += 1
    else:
        month += 1
    res = datetime.datetime(year, month, 1)
    return res


def get_date_list(start_date, end_date, fmt="%Y-%m-%d"):
    """根据给定的初始日期（‘2017-07-09’）来获取每一天日期的列表"""
    date_list = []
    start_date = datetime.datetime.strptime(start_date, fmt)
    end_date = datetime.datetime.strptime(end_date, fmt)
    while start_date <= end_date:
        date_str = start_date.strftime(fmt)
        date_list.append(date_str)
        start_date += datetime.timedelta(days=1)
    return date_list


def get_1st_of_last_week(index):
    """
    index:1 - 7 代表上周星期一到星期日
    获取上个月第一天的日期
    :return: 返回日期
    """
    if not index:
        return None
    now = datetime.datetime.now()
    date = now - datetime.timedelta(days=now.weekday() + (8 - index))
    return date


def get_date_of_week(index):
    """
    index:1 - 7 代表本周星期一到星期日
    :return: 返回日期
    """
    if not index:
        return None
    now = datetime.datetime.now()
    date = now - datetime.timedelta(days=now.weekday() - index + 1)
    return date


def get_flday_of_month(year=None, month=None):
    """获取某年某月的第一天和最后一天"""
    year = int(year) if year else datetime.date.today().year
    month = int(month) if month else datetime.date.today().month

    # 获取当月第一天的星期和当月的总天数
    fd_of_week, month_range = calendar.monthrange(year, month)

    # 获取当月的第一天
    fd_of_month = datetime.datetime(year=year, month=month, day=1)
    ld_of_month = datetime.datetime(year=year, month=month, day=month_range)

    return fd_of_month, ld_of_month


def is_valid_datetime(dt_str, fmt=DATETIME_FORMAT):
    """字符转换成时间，有转换成功标志"""
    valid = True
    result = None
    try:
        if not dt_str:
            valid = False
        else:
            result = datetime.datetime.strptime(dt_str, fmt)
    except ValueError:
        valid = False
    return valid, result


def format_datetime_2date(dt):
    """
    将mongodb保存的日期时间格式转为日期格式
    :param dt: mongodb保存的datetime日期时间格式
    :return: 返回转换后的日期格式（datetime.date类型）
    """
    if not dt:
        return None
    return datetime.date(dt.year, dt.month, dt.day)


def calculate_interval_days(begin_date, end_date=datetime.date.today()):
    """
    计算给定的两个日期间隔天数,默认计算给定天数到今天的间隔
    :param begin_date: 开始时间（datetime.date类型）
    :param end_date: 结束时间（datetime.date类型）
    :return: 返回间隔天数（int类型）
    """
    if begin_date is not None and begin_date <= end_date:
        delta = (end_date - begin_date).days
    else:
        delta = 0
    return delta



'''
Python 版本snowflake 雪花算法
Python version：3.9
来源于:https://blog.csdn.net/yanxilou/article/details/119645290
'''

import time

import yaoysTools.log.logutil as logutil

# 64位ID的划分
WORKER_ID_BITS = 5
DATACENTER_ID_BITS = 5
SEQUENCE_BITS = 12

# 最大取值计算
MAX_WORKER_ID = -1 ^ (-1 << WORKER_ID_BITS)  # 2**5-1 0b11111
MAX_DATACENTER_ID = -1 ^ (-1 << DATACENTER_ID_BITS)

# 移位偏移计算
WOKER_ID_SHIFT = SEQUENCE_BITS
DATACENTER_ID_SHIFT = SEQUENCE_BITS + WORKER_ID_BITS
TIMESTAMP_LEFT_SHIFT = SEQUENCE_BITS + WORKER_ID_BITS + DATACENTER_ID_BITS
# 序号循环掩码
SEQUENCE_MASK = -1 ^ (-1 << SEQUENCE_BITS)
# Twitter元年时间戳
TWEPOCH = 1288834974657

logger = logutil.getLogger(log_name='[snowflake]', log_level=logutil.MY_LOG_INFO)


class snowFlakeId(object):
    """
    用于生成IDs
    """

    def __init__(self, datacenter_id, worker_id, sequence=0):
        """
        初始化
        :param datacenter_id: 数据中心（机器区域）ID
        :param worker_id: 机器ID
        :param sequence: 其实序号
        """
        # sanity check
        if worker_id > MAX_WORKER_ID or worker_id < 0:
            raise ValueError('worker_id值越界')

        if datacenter_id > MAX_DATACENTER_ID or datacenter_id < 0:
            raise ValueError('datacenter_id值越界')

        self.worker_id = worker_id
        self.datacenter_id = datacenter_id
        self.sequence = sequence

        self.last_timestamp = -1  # 上次计算的时间戳

    def _gen_timestamp(self):
        """
        生成整数时间戳
        :return:int timestamp
        """
        return int(time.time() * 1000)

    def get_id(self):
        """
        获取新ID
        :return:
        """
        timestamp = self._gen_timestamp()

        # 时钟回拨
        if timestamp < self.last_timestamp:
            logger.error('clock is moving backwards. Rejecting requests until {}'.format(self.last_timestamp))
            raise

        if timestamp == self.last_timestamp:
            self.sequence = (self.sequence + 1) & SEQUENCE_MASK
            if self.sequence == 0:
                timestamp = self._til_next_millis(self.last_timestamp)
        else:
            self.sequence = 0

        self.last_timestamp = timestamp

        new_id = ((timestamp - TWEPOCH) << TIMESTAMP_LEFT_SHIFT) | (self.datacenter_id << DATACENTER_ID_SHIFT) | (self.worker_id << WOKER_ID_SHIFT) | self.sequence
        return new_id

    def _til_next_millis(self, last_timestamp):
        """
        等到下一毫秒
        """
        timestamp = self._gen_timestamp()
        while timestamp <= last_timestamp:
            timestamp = self._gen_timestamp()
        return timestamp

# def get_id():
#     worker = snowFlakeId(1, 2, 0)
#     for x in range(1, 10):
#         # print(worker.get_id())
#         logutil.log_info(worker.get_id(), my_logger=logger)
#
#
# if __name__ == '__main__':
#     get_id()



from yaoysTools.log.logutil import getLogger
from yaoysTools.log.logutil import log_info
from yaoysTools.log.logutil import log_error
from yaoysTools.log.logutil import log_warn
from yaoysTools.log.logutil import log_debug, get_log_level



# -*- coding: UTF-8 -*-
"""
    作者: 小肥巴巴
    简书: https://www.jianshu.com/u/db796a501972
    邮箱: imyunshi@163.com
    github: https://github.com/xiaofeipapa/python_example
    您可以任意转载, 恳请保留我作为原作者, 谢谢.

    2022-4-21
    yaoys 在原作者代码的基础上进行优化修改
"""
from timeit import default_timer

import pymysql
import pymysql.cursors
from dbutils.pooled_db import PooledDB


class DMysqlConfig:
    """

        :param mincached:连接池中空闲连接的初始数量
        :param maxcached:连接池中空闲连接的最大数量
        :param maxshared:共享连接的最大数量
        :param maxconnections:创建连接池的最大数量
        :param blocking:超过最大连接数量时候的表现，为True等待连接数量下降，为false直接报错处理
        :param maxusage:单个连接的最大重复使用次数
        :param setsession:optional list of SQL commands that may serve to prepare
            the session, e.g. ["set datestyle to ...", "set time zone ..."]
        :param reset:how connections should be reset when returned to the pool
            (False or None to rollback transcations started with begin(),
            True to always issue a rollback for safety's sake)
        :param host:数据库ip地址
        :param port:数据库端口
        :param db:库名
        :param user:用户名
        :param passwd:密码
        :param charset:字符编码
    """

    def __init__(self, host, db, user, password, port):
        self.host = host
        self.port = port
        self.db = db
        self.user = user
        self.password = password

        self.charset = 'UTF8'  # 不能是 utf-8
        self.minCached = 10
        self.maxCached = 20
        self.maxShared = 10
        self.maxConnection = 100

        self.blocking = True
        self.maxUsage = 100
        self.setSession = None
        self.reset = True


# ---- 用连接池来返回数据库连接
class DMysqlPoolConn:
    __pool = None

    def __init__(self, config):
        if not self.__pool:
            self.__class__.__pool = PooledDB(creator=pymysql,
                                             maxconnections=config.maxConnection,
                                             mincached=config.minCached,
                                             maxcached=config.maxCached,
                                             maxshared=config.maxShared,
                                             blocking=config.blocking,
                                             maxusage=config.maxUsage,
                                             setsession=config.setSession,
                                             charset=config.charset,
                                             host=config.host,
                                             port=config.port,
                                             database=config.db,
                                             user=config.user,
                                             password=config.password,
                                             )

    def get_conn(self):
        return self.__pool.connection()


# ---- 用pymysql 操作数据库
def get_connection(host, port, db, user, password):
    db_config = DMysqlConfig(host, db, user, password, port)
    g_pool_connection = DMysqlPoolConn(db_config)
    conn = g_pool_connection.get_conn()
    return conn


# ---- 使用 with 的方式来优化代码
class UsingMysql(object):

    def __init__(self, host, port, dbname, username, password, commit=True, log_time=True, log_label='总用时'):
        """

        :param commit: 是否在最后提交事务(设置为False的时候方便单元测试)
        :param log_time:  是否打印程序运行总时间
        :param log_label:  自定义log的文字
        :param host: 数据库ip地址
        :param port: 端口号
        :param dbname: 库名
        :param username: 数据库用户名
        :param password: 数据库密码
        """
        self._log_time = log_time
        self._commit = commit
        self._log_label = log_label
        self._host = host
        self._port = port
        self._dbname = dbname
        self._username = username
        self._password = password

    def __enter__(self):

        # 如果需要记录时间
        if self._log_time is True:
            self._start = default_timer()

        # 在进入的时候自动获取连接和cursor
        conn = get_connection(self._host, self._port, self._dbname, self._username, self._password)
        cursor = conn.cursor(pymysql.cursors.DictCursor)
        conn.autocommit = False

        self._conn = conn
        self._cursor = cursor
        return self

    def __exit__(self, *exc_info):
        # 提交事务
        if self._commit:
            self._conn.commit()
        # 在退出的时候自动关闭连接和cursor
        self._cursor.close()
        self._conn.close()

        if self._log_time is True:
            diff = default_timer() - self._start
            print('-- %s: %.6f 秒' % (self._log_label, diff))

    # 查找一条数据
    def fetch_one(self, sql, params=None):
        self.cursor.execute(sql, params)
        return self.cursor.fetchone()

    # 查找所有数据
    def fetch_all(self, sql, params=None):
        self.cursor.execute(sql, params)
        return self.cursor.fetchall()

    # 根据id获取数据
    def fetch_by_pk(self, sql, pk):
        self.cursor.execute(sql, (pk,))
        return self.cursor.fetchall()

    # 根据参数更新数据
    def update_all(self, sql, params=None):
        self.cursor.execute(sql, params)

    # 根据参数删除数据
    def delete_all(self, sql, params=None):
        self.cursor.execute(sql, params)

    @property
    def cursor(self):
        return self._cursor


"""
以下为测试
"""

# def delete_one(cursor, name):
#     sql = 'delete from user where name = %s'
#     params = name
#     cursor.execute(sql, params)
#     print('--- 已删除名字为%s. ' % name)
#
#
# def select_one(cursor):
#     sql = 'select * from user'
#     cursor.execute(sql)
#     return cursor.fetchone()
#
#
# def update_by_pk(cursor, name, pk):
#     sql = "update user set name = '%s' where id = %d" % (name, pk)
#
#     cursor.execute(sql)
#
#
# # 修改记录
# def check_update(cursor):
#     # 查找一条记录
#     data = select_one(cursor)
#     pk = data['id']
#     print('--- {0}: '.format(data))
#
#     # 修改名字
#     new_name = 'aaaa'
#     update_by_pk(cursor, new_name, pk)
#
#     # 查看
#     select_one_by_name(cursor, new_name)
#
#
# def select_one_by_name(cursor, name):
#     sql = 'select * from user where name = %s'
#     params = name
#     cursor.execute(sql, params)
#     data = cursor.fetchone()
#     if data:
#         print('--- 已找到名字为%s. ' % data['name'])
#     else:
#         print('--- 名字为%s的已经没有了' % name)
#
#
# if __name__ == '__main__':
#     host = 'localhost'
#     port = 3306
#     db = 'test'
#     user = 'root'
#     password = '123456'
#     with UsingMysql(host=host, port=port, dbname=db, username=user, password=password, log_time=True) as um:
#         um.cursor.execute("select count(id) as total from user")
#         data = um.cursor.fetchone()
#         print("-- 当前数量: %d " % data['total'])
#
#         um.cursor.execute("select * from user")
#         data = um.cursor.fetchone()
#         print("-- 单条记录: {0} ".format(data))
#
#     with UsingMysql(host=host, port=port, dbname=db, username=user, password=password, log_time=True) as um:
#         sql = "insert into user(name, age) values(%s, %s)"
#         params = ("user111", 25)
#         um.cursor.execute(sql, params)
#
#         um.cursor.execute("select * from user")
#         data = um.cursor.fetchall()
#         print("-- 记录: {0} ".format(data))



import setuptools  # 导入setuptools打包工具
from pathlib import Path

cwd = Path(__file__).parent
long_description = (cwd / "README.MD").read_text(encoding='utf8')
"""
打包命令: python setup.py sdist
twine upload dist/yaoys-python-tool-0.0.36.tar.gz
"""

setuptools.setup(
    name="yaoys-python-tool",  # pip库名称
    version="0.0.44",  # 包版本号，便于维护版本
    author="YaoYuanshuai",  # 作者
    author_email="yys9508@qq.com",  # 联系方式
    description="python工具类",  # 包的简述
    # long_description="此工具类致力于封装一些常用的python代码，目前支持log日志，progress进度条，mysql数据库操作，雪花算法唯一id，时间工具类等方法",  # 包的详细介绍，一般在README.md文件内
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/yaoysyao/PythonTools.git",  # 项目地址
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],

    # 对python的最低版本要求
    python_requires='>=3.6',
    install_requires=['pymysql',
                      'dbutils>=3.0.2',
                      'sklearn',
                      'numpy>=1.20.3',
                      'threadpoolctl>=2.0.0',
                      'joblib>=0.11',
                      'colorlog>=4.0.0'],
    # data_files=list(str(filepath))
    include_package_data=True,
    exclude_package_date={'': ['.gitignore']}
)



import numpy as np
from sklearn.model_selection import train_test_split


# 自定义划分测试数据集与训练数据集
def split_train_test(data, test_ratio=0.2):
    """
    :param
    data: 数据
    test_ratio:测试数据集的比例，默认0.2，按照2-8原则划分
    """
    # 设置随机数种子，保证每次生成的结果都是一样的
    np.random.seed(42)
    # permutation随机生成0-len(data)随机序列
    shuffled_indices = np.random.permutation(len(data))
    # test_ratio为测试集所占的半分比
    test_set_size = int(len(data) * test_ratio)
    test_indices = shuffled_indices[:test_set_size]
    train_indices = shuffled_indices[test_set_size:]
    # iloc选择参数序列中所对应的行
    return data.iloc[train_indices], data.iloc[test_indices]


# 使用sklearn库函数
def split_train_test_sklearn(data, test_ration=0.2, train_ration=0.8, random_state=42):
    return train_test_split(data, test_size=test_ration, train_size=train_ration, random_state=random_state)
